﻿'**************************************************
'  USB-ADC付属基板使用XBeeワイヤレスデータロガー
'  操作テストプログラム
'**************************************************
'   V0.10   2011/01/31
'   V1.00   2011/02/09
'
Imports System.IO.Ports

Public Class Form1

    Delegate Sub SetTextCallback(ByVal [text] As String)

    Dim timer_state As Integer = 0  ' Timerステート変数

    'シリアルポートリストを更新するためのタイマイベント
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        UpdateCOMPortList()  'シリアルポートリストを更新
    End Sub

    '接続されているCOMポート番号のリストを作成。一定時間ごとに呼ばれる
    Private Sub UpdateCOMPortList()
        Dim s As String = ""
        Dim i As Integer
        Dim foundDifference As Boolean
        ' 接続ポート一覧を表示させるために、SeirialPortクラスを呼び出す
        Dim ports As String() = SerialPort.GetPortNames()

        i = 0
        foundDifference = False 'ポートに変化があったか？
        ' 前回リストと比較
        If cbxPortNo.Items.Count = ports.Length Then
            'シリアルポート接続の検索
            For Each s In ports
                If cbxPortNo.Items(i).Equals(s) = False Then
                    foundDifference = True
                End If
                i = i + 1
            Next s
        Else
            foundDifference = True
        End If

        '前回と同じなら抜け出す
        If foundDifference = False Then
            Exit Sub
        End If

        '変更ある時はリストクリア
        cbxPortNo.Items.Clear()
        'その後新しいリストを探索追加
        For Each s In ports
            cbxPortNo.Items.Add(s)
        Next s
        'Set the listbox to point to the first entry in the list
        If s = Nothing Then
            cbxPortNo.Text = ""   'ポートから外された場合
        Else
            cbxPortNo.SelectedIndex = 0
        End If
    End Sub

    'シリアル受信文字を２個のテキストボックスに格納
    ' tbxDataReceivedは表示用、tbxRecbufは非表示で処理用の受信バッファとして使用
    Private Sub SetText(ByVal [text] As String)
        If tbxDataReceived.InvokeRequired Then        '異なるスレッドか判定
            '異なるスレッドなら、SetTextCallbackデリゲートを通じてTextを渡す
            Dim d As New SetTextCallback(AddressOf SetText)

            'Invoke the new delegate sending the same text to the
            '  delegate that was passed into this function from the
            '  other thread.
            Invoke(d, New Object() {[text]})
        Else
            'If this function was called from the same thread that 
            '  holds the required objects then just add the text.
            tbxDataReceived.AppendText(text)
        End If
    End Sub

    ' 接続ボタンクリック
    Private Sub btnConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnConnect.Click
        ' リストから接続ポート名を得る
        SerialPort1.PortName = cbxPortNo.Items(cbxPortNo.SelectedIndex).ToString()

        'ここから先は、シリアルポートコンポーネントを使う
        SerialPort1.Open()

        btnConnect.Enabled = False
        cbxPortNo.Enabled = False

        '受信バッファクリア
        tbxDataReceived.Clear()
        btnConnect.Enabled = False
        cbxSetting.Enabled = True
    End Sub

    ' シリアルポート受信割り込み
    Private Sub SerialPort1_DataReceived(ByVal sender As System.Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort1.DataReceived
        Try
            SetText(SerialPort1.ReadExisting())
        Catch ex As Exception
        End Try
    End Sub


    ' コマンド表示領域初期化
    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        tm3cond = 0
        timer_state = 0
        lvwComm.View = View.Details
        lvwComm.Columns.Clear()
        lvwCommInit()
        ConfigFileRead("Init.cfg")  '初期設定ファイル読み出し

        lvwDataInit()               'データ領域カラム初期
    End Sub

    Private Sub lvwCommInit()
        lvwComm.Columns.Add("入力", 60)
        lvwComm.Columns.Add("No1", 60)
        lvwComm.Columns.Add("No2", 60)
        lvwComm.Columns.Add("No3", 60)

        lvwComm.Items.Add("AIN1 CF")
        lvwComm.Items.Add("AIN1 MD")
        lvwComm.Items.Add("AIN2 CF")
        lvwComm.Items.Add("AIN2 MD")
        lvwComm.Items.Add("AIN3 CF")
        lvwComm.Items.Add("AIN4 MD")
        lvwComm.Items.Add("TEMP CF")
        lvwComm.Items.Add("TEMP MD")
    End Sub

    ' データ表示枠初期化
    Private Sub lvwDataInit()
        lvwData.Clear()
        lvwData.View = View.Details
        lvwData.Columns.Clear()
        lvwData.Columns.Add("No", 30)
        lvwData.Columns.Add("1-AIN1", 60)
        lvwData.Columns.Add("1-AIN2", 60)
        lvwData.Columns.Add("1-AIN3", 60)
        lvwData.Columns.Add("1-Temp", 60)
        lvwData.Columns.Add("2-AIN1", 60)
        lvwData.Columns.Add("2-AIN2", 60)
        lvwData.Columns.Add("2-AIN3", 60)
        lvwData.Columns.Add("2-Temp", 60)
        lvwData.Columns.Add("3-AIN1", 60)
        lvwData.Columns.Add("3-AIN2", 60)
        lvwData.Columns.Add("3-AIN3", 60)
        lvwData.Columns.Add("3-Temp", 60)

    End Sub

    Dim tm3cond As Integer = 0
    Dim lvwdatanum As Integer = 0
    Const MAXTERM As Integer = 3

    ' 100msごとのメインタイムベース
    ' 測定中は最短インターバル
    Private Sub Timer3_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer3.Tick
        Static Dim dtno, iNo As Integer
        Static Dim dtnob As Integer = -1
        Dim estrg As String
        Dim tickCount As Integer = System.Environment.TickCount And Integer.MaxValue
        Static Dim tkcnt, tkcntb, tkcntdeff As Integer
        Static Dim samptimes As Integer

        'サーチステート
        If timer_state = 1 Then
            If tm3cond = 10 Then
                terminal_init(1)
                tbxDataReceived.Clear()
            End If
            If tm3cond = 12 Then
                If tbxDataReceived.Lines.Count >= 1 Then
                    If Strings.Left(tbxDataReceived.Lines(0), 2) = "@1" Then
                        cbxNo1.Checked = True
                    End If
                End If
            End If
            If tm3cond = 20 Then
                terminal_init(2)
                tbxDataReceived.Clear()
            End If
            If tm3cond = 22 Then
                If tbxDataReceived.Lines.Count >= 1 Then
                    If Strings.Left(tbxDataReceived.Lines(0), 2) = "@2" Then
                        cbxNo2.Checked = True
                    End If
                End If
            End If
            If tm3cond = 30 Then
                terminal_init(3)
                tbxDataReceived.Clear()
            End If
            If tm3cond = 32 Then
                If tbxDataReceived.Lines.Count >= 1 Then
                    If Strings.Left(tbxDataReceived.Lines(0), 2) = "@3" Then
                        cbxNo3.Checked = True
                    End If
                End If
            End If
            If tm3cond > 32 Then        '３台目サーチ終了後は測定に入る
                timer_state = 2
                Timer3.Interval = 10    '測定はtickcountで行うので、インターバルは最小
                tm3cond = 0
            End If
            tm3cond = tm3cond + 1
        End If

        If timer_state = 2 Then         '測定ステート遷移のための中間ステート
            tkcntb = tickCount
            samptimes = 0
            timer_state = 3
            dtno = 0
        End If

        '測定ステート
        If timer_state = 3 Then
            Dim sampnum As Integer

            sampnum = nudSampTime.Value * 1000   'ms
            tkcnt = tickCount                    '前回よりsamptime以上経過?
            tkcntdeff = tkcnt - tkcntb           '経過時間取得
            If (tkcntdeff >= sampnum * samptimes) Then
                samptimes = samptimes + 1
                lvwData.Items.Add(dtno)                     '行追加
                For i = 0 To 11
                    lvwData.Items(dtno).SubItems.Add("")    '空きセル作成
                Next
                'TextBox1.Text = CStr(tickCount)
                dtno = dtno + 1
            End If
            If tbxDataReceived.Lines.Count > 1 Then
                estrg = tbxDataReceived.Lines(0)
                If (Strings.Len(estrg) = 26) And (dtno > 0) Then
                    'tkcntdeff = tkcnt - tkcntb              '受信時の経過時間取得
                    Dim dtdeff As Integer
                    iNo = CInt(Strings.Mid(estrg, 2, 1)) - 1    '機種No取得
                    For i = 0 To 3
                        lvwData.Items(dtno - 1).SubItems(iNo * 4 + 1 + i).Text = Strings.Mid(estrg, 6 * i + 3, 6)
                    Next
                    dtdeff = ((tkcntdeff Mod sampnum) - ((iNo + 1) * 1000)) / 10    '時間差分を10ms単位で計算
                    TextBox1.Text = CStr(dtdeff)
                    If dtdeff >= 127 Then dtdeff = 127
                    If dtdeff <= -128 Then dtdeff = -128
                    SerialPort1.Write("@" + CStr(iNo + 1) + Strings.Right("00" + Hex(dtdeff), 2) + vbCrLf)
                    While (SerialPort1.BytesToWrite <> 0) : End While
                End If
                tbxDataReceived.Clear()
            End If
        End If
    End Sub

    '端末初期化文字列送信
    ' no:端末番号(0..2)
    Private Sub terminal_init(ByVal no As Integer)
        Dim strg As String
        Dim i As Integer

        strg = "@" + CStr(no)
        For i = 0 To 7
            strg = strg + Strings.Right(lvwComm.Items(i).SubItems(no).Text, 6)
        Next
        strg = strg + Strings.Right("00" + Hex(nudSampTime.Value), 2) + vbCrLf
        SerialPort1.Write(strg)
    End Sub

    ' 探索ボタンクリック
    Private Sub btnSearch_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSearch.Click
        cbxNo1.Checked = False
        cbxNo2.Checked = False
        cbxNo3.Checked = False
        lvwData.Clear()
        lvwDataInit()
        Timer3.Enabled = True
        tm3cond = 0
        timer_state = 1
        btnSearch.Enabled = False
        btnStop.Enabled = True
        Timer3.Interval = 100
    End Sub

    ' 停止ボタンクリック
    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click
        btnStop.Enabled = False
        Timer3.Enabled = False
        tm3cond = 0
        timer_state = 0
        btnSearch.Enabled = True
        btnDataSave.Enabled = True
    End Sub

    Dim FileName As String
    Dim FileNum As Integer
    Dim CurrentDir As String

    '設定ファイル読み出しボタンクリック
    Private Sub btnSetLoad_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSetLoad.Click
        OpenFileDialog1.DefaultExt = "cfg"
        OpenFileDialog1.Filter = "設定ファイル(*.cfg)|*.cfg"
        OpenFileDialog1.InitialDirectory = CurrentDir
        If OpenFileDialog1.ShowDialog = DialogResult.OK Then
            FileName = OpenFileDialog1.FileName
        Else
            Exit Sub
        End If
        ConfigFileRead(FileName)
    End Sub

    '設定ファイル読み出し
    Private Sub ConfigFileRead(ByVal Fname As String)
        Dim strg As String
        Dim tmpTextBox As New TextBox
        Dim i As Integer

        tmpTextBox.Clear()
        tmpTextBox.Multiline = True

        FileNum = FreeFile()
        FileOpen(FileNum, Fname, OpenMode.Input)
        'Input(FileNum, tmpTextBox.Text)
        Do Until EOF(FileNum)
            strg = LineInput(FileNum)
            If (Strings.Left(Strings.Trim(strg), 1) <> ";") And (Strings.Trim(strg) <> "") Then    'コメント行?
                tmpTextBox.AppendText(strg + vbCrLf)
            End If
        Loop
        FileClose(FileNum)
        If tmpTextBox.Lines.Count <> 25 Then
            MsgBox(Fname + "は設定ファイルではありません" + CStr(tmpTextBox.Lines.Count), MsgBoxStyle.OkOnly, "ERROR")
        End If
        lvwComm.Clear()
        lvwCommInit()
        Try
            For i = 0 To 7
                lvwComm.Items(i).SubItems.Add(Strings.Left(tmpTextBox.Lines(i), 6))
            Next i
            For i = 0 To 7
                lvwComm.Items(i).SubItems.Add(Strings.Left(tmpTextBox.Lines(i), 6))
            Next i
            For i = 0 To 7
                lvwComm.Items(i).SubItems.Add(Strings.Left(tmpTextBox.Lines(i), 6))
            Next i
        Catch ex As Exception

        Finally
            tmpTextBox.Dispose()
        End Try

    End Sub

    '測定データ保存
    Private Sub btnDataSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDataSave.Click
        Dim i, j, bufcnt As Integer
        Dim tmpTextBox As New TextBox
        Dim strg As String
        Try
            tmpTextBox.Clear()
            tmpTextBox.Multiline = True
            'コメント行
            bufcnt = lvwData.Items.Count - 1
            For i = 0 To bufcnt
                strg = ""
                For j = 1 To 4
                    strg = strg + """" + Strings.Right("000000" + lvwData.Items(i).SubItems(j).Text, 6) + """" + ","
                Next j
                tmpTextBox.AppendText(strg + vbCrLf)
            Next i
            SaveFileDialog1.DefaultExt = "txt"
            SaveFileDialog1.Filter = "データファイル(*.csv)|*.csv"
            SaveFileDialog1.FileName = "WLG" + DateTime.Now.ToString("yyyyMMddHHmm") + ".csv"
            If SaveFileDialog1.ShowDialog = DialogResult.OK Then
                FileName = SaveFileDialog1.FileName
            Else
                Exit Sub
            End If
            FileNum = FreeFile()
            FileOpen(FileNum, FileName, OpenMode.Output)
            Print(FileNum, tmpTextBox.Text)
        Catch ex As Exception
            MsgBox("不明なファイルエラー", MsgBoxStyle.OkOnly, "エラー")
        Finally
            FileClose(FileNum)
            tmpTextBox.Dispose()
        End Try
    End Sub
End Class
